//This code conditions on a fixed interest rate path and gives it two interpretation:
//The fixed interest rate is actual policy, so it is interpreted as a temporary deviation from the Taylor rule.
//The fixed interest rate is assumed to happen 'by accident', so the Kalman smoother searches for the most likely
//configuration of shocks.
//By making the variance of the monetary policy shock the only variance (do this by setting the 
//    zero for our purposes. Note that when you implement the fixed rate policy in this way, the negative effect on 
//    output is smaller because the fixed rate shock is actually a surprise to people. 


var da pie r rstar tau u x; 	  

varexo eps_a eps_tau eps_u;   

parameters phi beta kappa phi_x phi_pie alpha rho lambda delta; // DECLARATION OF THE DEEP PARAMETERS.

// Parameter Values  
beta    = 0.99;
phi_x   = 0.;
phi_pie = 1.5;
alpha   = 0.85;
rho     = 0.9;
lambda  = 0.7;
delta   = 0.0;
phi     = 1;
theta   = 0.75;
kappa   = ((1-theta)*(1-beta*theta)*(1+phi))/(theta);

// DECLARATION OF THE (LINEAR) DSGE MODEL: 
model(linear);

    beta*pie(+1) + kappa*x = pie;                                      // Calvo Pricing Equation (Eqn 1)
    r - pie(+1)-rstar   = x(+1) - x;                                   // Intertemporal Equation (Eqn 2)
    alpha*r(-1) + u + (1-alpha)*phi_pie*pie + (1-alpha)*phi_x*x = r;   // Monetary Policy Rule (Eqn 3)
    rstar = rho*da + (1-lambda)/(1+phi)*tau;                           // Definition of the Natural Rate (Eqn 4)
    da = rho*da(-1) + eps_a;                                           // Exogenous process for da (Eqn 5)
    tau       = lambda*tau(-1) + eps_tau;                              // Exogenous process for tau (Eqn 6)
    u         = delta*u(-1) + eps_u;                                   // Exogenous process for u (Eqn 7)

end;


shocks;
var eps_a;
stderr .001;
var eps_tau;
stderr .001;
var eps_u;
stderr .1;
end;

steady;
check;

set_dynare_seed(1);

stoch_simul(irf=7,periods=516,nograph) da pie r rstar tau u x;

%first, compute the fixed interest rate simulation....

%Row vector of interest rate increases (decreases if negative) to add to the last
%period observation in the data set. Because the algorithm solves the model 'backwards' through
%time, the interest rate path must begin with the last entry, then the next-to-last, etc.

FixInterestRateVect = 0.01*ones(1,4);
\\FixInterestRateVect = [.01 0.*ones(1,3)];

%Length of the whole forecast horizon including the period with the known interest rate path
forelength = 16;
%give the name (in the var command) of the nominal rate of interest
PolicyRateName = 'r'; 
%give the name (in the varexo command) of the innovation in the shock to montary policy
MonetaryShockName = 'eps_u';
%which equation in the mod file corresponds to the monetary policy equation
EqnR = 3;
%save the information you just entered...
save ForeDataParams FixInterestRateVect forelength PolicyRateName MonetaryShockName EqnR;

%now, compute the conditional forecasts...
ConditionalForecasts;

%now, graph the results
close all;
figure_setup;
fig=figure('name','unconstrained, Odysssean Fixed Rate forecast');

for iii = 1:length(oo_.dr.order_var)
ttt = (-8:1:forelength)';        
subplot(4,2,iii);
plot(ttt,dataactfore(size(data,1)-8:end,iii),'k', ...
    ttt,ConditionalForecast(size(data,1)-8:end,iii),'b-o',...
    ttt,UnConditionalForecast(size(data,1)-8:end,iii),'m--')
xlim([-8 16])

grid on;
title(M_.endo_names(oo_.dr.order_var(iii),:))

end

subplot(4,2,8)
plot(nan,nan,'k',nan,nan,'b-o',nan,nan,'m--');set(gca,'Visible','off')
legend('Pre-Simulation Data',...
         'Odyssean Fixed Interest Rate Forecast',...
         'Unconditional Forecast','Location','southoutside');legend boxoff
        legend boxoff 
      
   
//to print this figure out as a pdf, uncomment the following line:
//save_pdf(fig,'figure_for_handout.pdf',2);

//return

%now compute the forecast of the variables, conditional on the fixed interest rate path... 
%the algorithm requires the interest rate path in right order through time and
%above, the entries in FixInterestRateVect occur in reverse order through time:
FixInterestRateVect=FixInterestRateVect(end:-1:1);
%resave the material with the alteration in FixInterestRateVect...
save ForeDataParams FixInterestRateVect forelength PolicyRateName MonetaryShockName EqnR;

%first, set up the 'future' data to include the interest rate path on which we're conditioning...
set_data;

%now, get the estimation command (with mode_compute=0, so that no mode estimation is done) to compute the
%forecast of the other variables, conditional on the interest rate being equal to the fixed interest rate path studied 
%above
varobs r x pie;
estimation(datafile=CGGsimuldata,mode_compute=0,mh_replic=0,smoother,smoothed_state_uncertainty) da pie r rstar tau u x;

%now, get the information produced by the estimation command so that the results can be graphed.
ConditionalSmoothedMat = [];
for iii = 1:size(M_.endo_names,1)
    eval(['smoothvar = oo_.SmoothedVariables.' M_.endo_names(iii,:) ';']);
    ConditionalSmoothedMat = [ConditionalSmoothedMat smoothvar];
end
ConditionalSmoothedMat = ConditionalSmoothedMat(:,oo_.dr.order_var);
ConditionalSmoothedMat(1:insampsize-1,:) = NaN;

fig=figure('name','unconstrained, Odysssean Fixed Rate forecast, Smoothed forecast');


for iii = 1:length(oo_.dr.order_var)
ttt = (-8:1:forelength)';        
subplot(4,2,iii);
plot(ttt,dataactfore(size(data,1)-8:end,iii),'k',ttt,ConditionalSmoothedMat(size(data,1)-8:end,iii),'r-*',...
    ttt,ConditionalForecast(size(data,1)-8:end,iii),'b-o',...
    ttt,UnConditionalForecast(size(data,1)-8:end,iii),'m--')
xlim([-8 16])

grid on;
title(M_.endo_names(oo_.dr.order_var(iii),:))


end

subplot(4,2,8)
plot(nan,nan,'k',nan,nan,'r-*',nan,nan,'b-o',nan,nan,'m--');set(gca,'Visible','off')
legend('Pre-Simulation Data','Smoothed Conditional Interest Rate Forecast (i.e., ''Surprise Policy'')',...
         'Odyssean Fixed Interest Rate Forecast (''Full Transparency Policy'')',...
         'Unconditional Forecast');
        legend boxoff 

//to print this figure out as a pdf, uncomment the following line:
//save_pdf(fig,'figure_for_handout_compare.pdf',2);


